==> osam_flutter-2.2.6.tar.gz: lib/src/presentation/osam_provider.dart <==
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:osam/osam.dart';
import 'package:provider/provider.dart';
import 'package:provider/single_child_widget.dart';

part 'presenter.dart';

abstract class OsamProvider implements Widget {
  const factory OsamProvider(
      {UseCaseContainer useCaseContainer,
      Object appState,
      Object uiState,
      int version,
      Key key,
      Widget child,
      Persist<Object, Object> persist}) = _OsamProvider;
}

class _OsamProvider extends InheritedWidget implements OsamProvider {
  const _OsamProvider(
      {this.useCaseContainer, this.appState, this.uiState, this.version, Key key, Widget child, this.persist})
      : super(key: key, child: child);

  final Object appState;
  final UseCaseContainer useCaseContainer;
  final Object uiState;
  final Persist<Object, Object> persist;
  final int version;

  static _OsamProvider of(BuildContext context) => context.dependOnInheritedWidgetOfExactType(aspect: _OsamProvider);

  @override
  bool updateShouldNotify(_OsamProvider oldWidget) => false;
}

class PersistLifecycleWrapper extends StatefulWidget {
  final Widget child;
  final bool automaticHideKeyboardOnPause;

  const PersistLifecycleWrapper({Key key, this.child, this.automaticHideKeyboardOnPause = false}) : super(key: key);

  @override
  _PersistLifecycleWrapperState createState() => _PersistLifecycleWrapperState();
}

class _PersistLifecycleWrapperState extends State<PersistLifecycleWrapper> with WidgetsBindingObserver {
  @override
  void initState() {
    WidgetsBinding.instance.addObserver(this);
    super.initState();
  }

  @override
  Widget build(BuildContext context) => widget.child;

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    if (state == AppLifecycleState.paused) {
      if (widget.automaticHideKeyboardOnPause) {
        SystemChannels.textInput.invokeMethod('TextInput.hide');
      }
      final useCaseProvider = _OsamProvider.of(context);
      useCaseProvider.persist
          .save(state: useCaseProvider.appState, uiState: useCaseProvider.uiState, version: useCaseProvider.version);
    }
  }
}

mixin _PresenterProviderSingleChildWidget on SingleChildWidget {}

class PresenterProvider<P extends Presenter> extends SingleChildStatelessWidget
    with _PresenterProviderSingleChildWidget {
  final Widget child;
  final P presenter;

  const PresenterProvider({Key key, this.child, this.presenter}) : super(key: key, child: child);

  static P of<P extends Presenter>(BuildContext context) => Provider.of<P>(context, listen: false);

  @override
  Widget buildWithChild(BuildContext context, Widget child) {
    final useCaseProvider = _OsamProvider.of(context);
    return Provider<P>(
      create: (_) {
        presenter?._useCaseContainer = useCaseProvider.useCaseContainer;
        presenter?._uiState = useCaseProvider?.uiState;
        presenter?.init();
        return presenter;
      },
      child: child,
      dispose: (ctx, presenter) => presenter?.dispose(),
    );
  }
}

class MultiPresenterProvider extends StatelessWidget {
  final List<SingleChildWidget> providers;
  final Widget child;

  const MultiPresenterProvider({Key key, @required this.providers, @required this.child}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return MultiProvider(
      providers: providers,
      key: ValueKey('PresenterMultiProvider: %${key.toString()}'),
      child: child,
    );
  }
}

extension PresenterProviderExtension on BuildContext {
  P presenter<P extends Presenter>() => PresenterProvider.of<P>(this);
}

==> osam_flutter-3.0.1.tar.gz: lib/src/presentation/osam_provider.dart <==
import 'package:flutter/widgets.dart';
import 'package:osam/osam.dart';
import 'package:provider/provider.dart';
import 'package:provider/single_child_widget.dart';

part 'presenter.dart';

abstract class OsamProvider implements Widget {
  const factory OsamProvider(
      {UseCaseContainer useCaseContainer,
      Object appState,
      Object uiState,
      int version,
      Key key,
      Widget child,
      Persist<Object, Object> persist}) = _OsamProvider;
}

class _OsamProvider extends InheritedWidget implements OsamProvider {
  const _OsamProvider(
      {this.useCaseContainer, this.appState, this.uiState, this.version, Key key, Widget child, this.persist})
      : super(key: key, child: child);

  final UseCaseContainer useCaseContainer;
  final Object appState;
  final Object uiState;
  final int version;
  final Persist<Object, Object> persist;

  static _OsamProvider of(BuildContext context) => context.dependOnInheritedWidgetOfExactType(aspect: _OsamProvider);

  @override
  bool updateShouldNotify(_OsamProvider oldWidget) => false;
}

class PersistLifecycleWrapper extends StatefulWidget {
  final Widget child;

  const PersistLifecycleWrapper({Key key, this.child}) : super(key: key);

  @override
  _PersistLifecycleWrapperState createState() => _PersistLifecycleWrapperState();
}

class _PersistLifecycleWrapperState extends State<PersistLifecycleWrapper> with WidgetsBindingObserver {
  @override
  void initState() {
    WidgetsBinding.instance.addObserver(this);
    super.initState();
  }

  @override
  Widget build(BuildContext context) => widget.child;

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    if (state == AppLifecycleState.paused) {
      final useCaseProvider = _OsamProvider.of(context);
      useCaseProvider.persist
          .save(state: useCaseProvider.appState, uiState: useCaseProvider.uiState, version: useCaseProvider.version);
    }
  }
}

mixin _PresenterProviderSingleChildWidget on SingleChildWidget {}

class PresenterProvider<P extends Presenter> extends SingleChildStatelessWidget
    with _PresenterProviderSingleChildWidget {
  final Widget child;
  final P presenter;

  const PresenterProvider({Key key, this.child, this.presenter}) : super(key: key, child: child);

  static P of<P extends Presenter>(BuildContext context) => Provider.of<P>(context, listen: false);

  @override
  Widget buildWithChild(BuildContext context, Widget child) {
    final useCaseProvider = _OsamProvider.of(context);
    return Provider<P>(
      create: (_) {
        presenter?._useCaseContainer = useCaseProvider.useCaseContainer;
        presenter?._uiState = useCaseProvider?.uiState;
        presenter?._appState = useCaseProvider?.appState;
        presenter?.init();
        return presenter;
      },
      child: child,
      dispose: (_, presenter) => presenter?.dispose(),
    );
  }
}

class MultiPresenterProvider extends StatelessWidget {
  final List<SingleChildWidget> providers;
  final Widget child;

  const MultiPresenterProvider({Key key, @required this.providers, @required this.child}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return MultiProvider(
      providers: providers,
      child: child,
    );
  }
}

extension PresenterProviderExtension on BuildContext {
  P presenter<P extends Presenter>() => PresenterProvider.of<P>(this);
}
